
TEST_SHIPS = [
  {
    "manufacturer": "Drake",
    "name": "Herald",
    "lti": True,
    "warbond": False,
    "package_id": "9635433",
    "pledge": "Standalone Ship - MPUV Personnel - LTI",
    "pledge_date": "August 30, 2016",
    "cost": "$85.00 USD"
  },
  {
    "manufacturer": "Drake",
    "name": "Cutlass Black",
    "lti": True,
    "warbond": True,
    "package_id": "12616268",
    "pledge": "Standalone Ship - Aopoa Nox Kue LTI  - Warbond",
    "pledge_date": "August 23, 2017",
    "cost": "$90.00 USD"
  },
  {
    "manufacturer": "RSI",
    "name": "Constellation Taurus",
    "lti": True,
    "warbond": False,
    "package_id": "12620007",
    "pledge": "Standalone Ship - MPUV Cargo - LTI",
    "pledge_date": "August 24, 2017",
    "cost": "$150.00 USD"
  }
]
